#climch_defs.py
#Functions for climate chamber control program climch*
#ver.14.3.2001

import math, time

def piddle(valuePrev,valueNow,targetNow,targetNext,multNow,multTrend):
    "Generic PID algorithm"
    errorNow = valueNow - targetNow #error in SI units, or whatever
    corrNow = errorNow * multNow #correction needed in volts, seconds &c
    #if math.fabs(errorNow) > 1: #1 unit proportional band
        #return (corrNow * 10) #move quickly to full heat or cool
    #else use proportional control + convergence control
    diverge = (valueNow - valuePrev) - (targetNext - targetNow)
    corrTrend = diverge * multTrend #correction for convergence
    print ' err %0.2f  div %0.2f' % (corrNow, corrTrend)
    return (corrNow + corrTrend)

def setTarget(elapsedTime,cycleTime,avg,halfAmp):
    "Value on sine curve"
    elapsedHours = elapsedTime/3600
    nix,hoursIntoCycle = divmod(elapsedHours,cycleTime)
    cycleFraction = hoursIntoCycle/cycleTime
    radians = cycleFraction * math.pi * 2
    return (avg + halfAmp * math.sin(radians))

def vToB(vv, a):
   'converts & sends analog voltage as binary stream'
   # limit voltage range
   if vv < -8.0: vv = -8.0
   if vv > 4.0: vv = 4.0
   v = abs(vv)
   sV = 30 + v*20 # convert v to 30-255 scale
   dd = 128
   for b in range(8):
      if sV > dd: # set one = 0 volts at d/a converter
         a.dialog('rout:open (@203)\n')
	 sV = sV - dd
      else:
         a.dialog('rout:clos (@203)\n')
      time.sleep(0.1)
      a.dialog('rout:close (@202)\n')#set clock
      time.sleep(0.1)
      a.dialog('rout:open (@202)\n')#clock to register
      dd = dd/2
   time.sleep(.1)
   a.dialog('rout:clos (@204)\n') # reg -> DAC
   time.sleep(0.1)
   a.dialog('rout:open (@204)\n') # latched
   # set polarity
   if vv < 0.0:
      a.dialog('rout:open (@205)\n')
   else:
      a.dialog('rout:clos (@205)\n')

def svp(t):
   "saturation vapour pressure in pa"
   import math
   ps = 610.78 * math.exp( t / ( t + 238.3 ) *17.2694 )
   return ps

def dpToRh(at, dt):
   "ambient t and dew point to rh%"
   sa = svp(at)
   sd = svp(dt)
   rh = 100 * sd/sa
   return rh

# Honeywell HIH-3602-A RH calculation, refined 3/2001
# requires signal v, t, assumes 5V supply
def H_RH(v,t):
    "Honeywell HIH RH calculation"
    rh = (v - 1)/(.0313 - 0.0000028 * t)
    return rh

# Honeywell HIH-3602-A thermistor calculation
#Rb=100kohm@25C NTC, B=4143
# R(t)=Rb exp(B/t-B/298.15)
#requires r in ohms, i think!
def thermistor(r):
    "100K thermistor calc."
    b = 4143 #beta = 4143K
    c = b/298.15
    t = b/(math.log(r/100000) + c)- 273
    return t
